#include <windows.h>
#include <stdlib.h>
#include "uart.h"

void Wait_COM(HWND, int);
LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);

char COM_PORT=2; // COM_2

/* -------------------------------------------------------------------- */

int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance,
		    PSTR szCmdLine, int iCmdShow)
     {
     static char szAppName[] = "ShowCOM2";
     HWND	 hwnd ;
     MSG	 msg ;
     WNDCLASSEX  wndclass ;

     wndclass.cbSize	    = sizeof(wndclass);
     wndclass.style	    = CS_HREDRAW|CS_VREDRAW;
     wndclass.lpfnWndProc   = WndProc;
     wndclass.cbClsExtra    = 0;
     wndclass.cbWndExtra    = 0;
     wndclass.hInstance     = hInstance;
     wndclass.hIcon	    = LoadIcon(NULL, IDI_APPLICATION);
     wndclass.hCursor	    = LoadCursor(NULL, IDC_ARROW);
     wndclass.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH);
     wndclass.lpszMenuName  = NULL;
     wndclass.lpszClassName = szAppName;
     wndclass.hIconSm	    = LoadIcon(NULL, IDI_APPLICATION);

     RegisterClassEx(&wndclass) ;
     hwnd=CreateWindow(szAppName,"COM2 connect to WDT-01, press any key to show WDT-01 output",
			  WS_OVERLAPPEDWINDOW,
			  CW_USEDEFAULT, CW_USEDEFAULT,
			  CW_USEDEFAULT, CW_USEDEFAULT,
			  NULL, NULL, hInstance, NULL) ;
     ShowWindow(hwnd,iCmdShow);
     UpdateWindow(hwnd);

     while (GetMessage(&msg, NULL, 0, 0))
	  {
	  TranslateMessage(&msg);
	  DispatchMessage(&msg);
	  }
     return msg.wParam;
     }

/* -------------------------------------------------------------------- */

LRESULT CALLBACK WndProc(HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
     {
     static int  cxChar, cyChar, cxClient, cyClient, cxBuffer, cyBuffer,
		 xCaret, yCaret ;
     static char Buf[80],c;
     HDC	 hdc;
     int	 i;
     TEXTMETRIC  tm;
     PAINTSTRUCT ps;
     WORD	 ret;

     switch (iMsg)
       {
       case WM_CREATE :
	    hdc=GetDC(hwnd);
	    SelectObject(hdc, GetStockObject(SYSTEM_FIXED_FONT));
	    GetTextMetrics(hdc, &tm);
	    cxChar=tm.tmAveCharWidth;
	    cyChar=tm.tmHeight;
	    ReleaseDC(hwnd, hdc);
	    return 0;
       case WM_SIZE :
	    cxClient=LOWORD(lParam);	     // window size in pixels
	    cyClient=HIWORD(lParam);
	    cxBuffer=max(1,cxClient/cxChar); // window size in characters
	    cyBuffer=max(1,cyClient/cyChar);
	    xCaret=yCaret=0;		     // set caret to upper left corner
	    if (hwnd == GetFocus()) SetCaretPos(0,0);
	    return 0;
       case WM_SETFOCUS :
	    CreateCaret(hwnd, NULL, cxChar, cyChar);
	    SetCaretPos(xCaret * cxChar, yCaret * cyChar);
	    ShowCaret(hwnd);
	    return 0;
       case WM_KILLFOCUS :
	    HideCaret(hwnd);
	    DestroyCaret();
	    return 0;
       case WM_CHAR :
	    for (i = 0 ; i < (int) LOWORD(lParam) ; i++)
	    {
	    switch (wParam)
	      {
	      case '\b' :                    // backspace
		if (xCaret > 0)
		{
		xCaret-- ;
		Buf[xCaret] = ' ';
		HideCaret(hwnd);
		hdc=GetDC(hwnd);
		SelectObject(hdc,GetStockObject (SYSTEM_FIXED_FONT));
		TextOut(hdc, xCaret * cxChar, yCaret * cyChar,Buf+xCaret,1);
		ShowCaret(hwnd);
		ReleaseDC(hwnd, hdc);
		}
		break ;
	      case '\r' :                    // carriage return
		Buf[xCaret] = 0;
		if (++yCaret == cyBuffer) yCaret=0;

		Wait_COM(hwnd,yCaret*cyChar);

		break ;
	      case '\x1B' :                  // escape
		InvalidateRect (hwnd, NULL, TRUE) ;
		xCaret=yCaret=0;
		break ;
	      default : 		     // character codes
		Buf[xCaret]=(char) wParam;
		HideCaret(hwnd);
		hdc=GetDC (hwnd);
		SelectObject(hdc,GetStockObject(SYSTEM_FIXED_FONT));
		TextOut(hdc,xCaret*cxChar,yCaret*cyChar,Buf+xCaret,1);
		ShowCaret(hwnd);
		ReleaseDC(hwnd, hdc);
		if (++xCaret == cxBuffer)
		   {
		   xCaret=0; if (++yCaret==cyBuffer) yCaret=0;
		   }
		break ;
	      }
	    }
	    SetCaretPos (xCaret * cxChar, yCaret * cyChar);
	    return 0;
       case WM_PAINT :
	    InvalidateRect(hwnd, NULL, TRUE);
	    hdc = BeginPaint (hwnd, &ps);
	    xCaret = 0 ; yCaret=0;
	    if (hwnd == GetFocus()) SetCaretPos(0,0);
	    EndPaint(hwnd, &ps);
	    return 0;
       case WM_DESTROY :
	    PostQuitMessage(0);
	    return 0 ;
       }
     return DefWindowProc(hwnd, iMsg, wParam, lParam);
     }

/* -------------------------------------------------------------------- */

void Wait_COM(HWND hwnd, int yy)
{
char cBuf[80],cShow[80],ch;
HDC hdc;
WORD ret,i,t;

hdc=GetDC(hwnd);
SelectObject(hdc,GetStockObject(SYSTEM_FIXED_FONT));

ret=OPEN_COM(COM_PORT,9600L,0);
if (ret!=1)
   {
   strcpy(cShow,"Open Com Error");
   goto ret_label;
   }

for (;;)
    {
    ret=UART_READ_RECEIVE_STATUS(COM_PORT);
    if (ret!=0)
       {
       UART_RECEIVE_1_CHAR(COM_PORT, &ch);
       }
    else break;
    }

t=0;
for (;;)
    {
    ret=UART_READ_RECEIVE_STATUS(COM_PORT); /* finished or timeout */
    if (ret!=0)
       {
       UART_RECEIVE_1_CHAR(COM_PORT, &ch); /* finished or timeout */
       if (ch==0x0d) break;
       t=0;
       }
    else Sleep(1);
    t++;
    if (t>100)
       {
       strcpy(cShow,"Time Out");
       goto ret_label;
       }
    }

t=0; i=0;
for (;;)
    {
    ret=UART_READ_RECEIVE_STATUS(COM_PORT); /* finished or timeout */
    if (ret!=0)
       {
       UART_RECEIVE_1_CHAR(COM_PORT, &ch); /* finished or timeout */
       if (ch==0x0d) break;
       t=0;
       cBuf[i]=ch; i++;
       }
    else Sleep(1);
    t++;
    if (t>100)
       {
       strcpy(cShow,"Time Out");
       goto ret_label;
       }
    }
    cBuf[i]=0;

    strcpy(cShow,"Receive string : ");
    strcat(cShow,cBuf);
ret_label:
    TextOut(hdc,1,yy,cShow,strlen(cShow));
    ReleaseDC (hwnd,hdc);
    CLOSE_COM(COM_PORT);
}

